package instance;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * Class that generates an instance of specified size by generating tasks with
 * random start and end stations and start and end times. Note that it can occur
 * that the tasks generated in this way cannot be covered by any feasible
 * duties, e.g. when the earliest task does not start at a depot.
 * 
 * @author B.T.C. van Rossum
 */
public class InstanceGenerator
{
	/**
	 * Generate a random instance with a specified number of tasks, depots, and
	 * stations.
	 * 
	 * @param random      random generator
	 * @param numTasks    number of tasks
	 * @param numDepots   number of depots
	 * @param numStations number of stations
	 * @return randomly generated instance of specified size
	 */
	public static Instance generateInstance(Random random, int numTasks, int numDepots, int numStations)
	{
		// Generate depots.
		List<Integer> depots = new ArrayList<>();
		for (int i = 0; i < numDepots; i++)
		{
			depots.add(i);
		}

		// Generate tasks.
		List<Task> tasks = new ArrayList<>();
		for (int i = 0; i < numTasks; i++)
		{
			int startTime = random.nextInt(Parameters.MAX_START_TIME);
			int endTime = startTime + Parameters.MIN_TASK_LENGTH
					+ random.nextInt(Parameters.MAX_TASK_LENGTH - Parameters.MIN_TASK_LENGTH);
			int startStation = random.nextInt(numStations);
			int endStation = random.nextInt(numStations);

			tasks.add(new Task(i, startTime, endTime, startStation, endStation));
		}

		// Return instance.
		return new Instance(depots, tasks);
	}
}
