package scripts;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Locale;
import java.util.Random;

import columnGeneration.ColumnGeneration;
import ilog.concert.IloException;
import instance.Instance;
import instance.InstanceGenerator;
import instance.Parameters;
import pricingProblem.CubicBreak;
import pricingProblem.QuadraticLogBreak;

public class Main
{
	public static void main(String[] args) throws IloException, IOException
	{
		// Parameters.
		Random random = new Random(11235);
		int[] numTasks =
		{ 100, 250, 500, 750, 1000, 1250 };
		int numRuns = 10;
		int numSolvers = 2;

		// Initialise output file.
		StringBuilder sb = new StringBuilder();
		sb.append("n;solver;timePricing;pricingIter;timePerIter\n");

		// Perform runs.
		for (int n : numTasks)
		{
			// Initialise array to store time spent pricing and number of pricing iterations
			// of each solver.
			double[][] statisticsPerSolver = new double[numSolvers][2];

			for (int i = 0; i < numRuns; i++)
			{
				// Generate a random instance of size n.
				Instance instance = InstanceGenerator.generateInstance(random, n,
						Parameters.NUM_DEPOTS,
						Parameters.NUM_STATIONS);

				// Solve this instance with each different pricing problem solver.
				ColumnGeneration.applyColumnGeneration(instance, new CubicBreak(instance), statisticsPerSolver[0]);
				ColumnGeneration.applyColumnGeneration(instance, new QuadraticLogBreak(instance),
						statisticsPerSolver[1]);
			}

			// Store the results for this instance size.
			for (int i = 0; i < numSolvers; i++)
			{
				double time = statisticsPerSolver[i][0];
				double iter = statisticsPerSolver[i][1];
				double timePerIter = ((double) time / iter);

				sb.append(n + ";" + i + ";" + String.format(Locale.forLanguageTag("nl-NL"), "%.3f",
						time) + ";"
						+ String.format(Locale.forLanguageTag("nl-NL"), "%.3f",
								iter)
						+ ";" + String.format(Locale.forLanguageTag("nl-NL"), "%.3f",
								timePerIter)
						+ "\n");
			}
		}

		// Write results to file.
		BufferedWriter bufferedWriter = new BufferedWriter(new FileWriter("results.csv"));
		bufferedWriter.write(sb.toString());
		bufferedWriter.close();
	}
}
